@@ -74,6 +74,7 @@
 
 static uint16_t cpu_freq_count = 0;
 static int set_batch_freq = -1;
+static int set_interactive_freq = -1;
 
 static struct cpu_freq_data {
 	uint8_t  avail_governors;
@@ -449,10 +450,18 @@ cpu_freq_cpuset_validate(stepd_step_rec_t *step)
 		else
 			set_batch_freq = 0;
 	}
+	if (set_interactive_freq == -1) {
+		if (xstrcasestr(slurm_conf.launch_params,
+				"interactive_step_set_cpu_freq"))
+			set_interactive_freq = 1;
+		else
+			set_interactive_freq = 0;
+	}
 
 	if (((step->step_id.step_id == SLURM_BATCH_SCRIPT) &&
 	     !set_batch_freq) ||
-	    (step->step_id.step_id == SLURM_INTERACTIVE_STEP) ||
+	    ((step->step_id.step_id == SLURM_INTERACTIVE_STEP) &&
+	     !set_interactive_freq) ||
 	    (step->step_id.step_id == SLURM_EXTERN_CONT))
 		return;
 
@@ -552,10 +561,18 @@ cpu_freq_cgroup_validate(stepd_step_rec_t *step, char *step_alloc_cores)
 		else
 			set_batch_freq = 0;
 	}
+	if (set_interactive_freq == -1) {
+		if (xstrcasestr(slurm_conf.launch_params,
+				"interactive_step_set_cpu_freq"))
+			set_interactive_freq = 1;
+		else
+			set_interactive_freq = 0;
+	}
 
 	if (((step->step_id.step_id == SLURM_BATCH_SCRIPT) &&
 	     !set_batch_freq) ||
-	    (step->step_id.step_id == SLURM_INTERACTIVE_STEP) ||
+	    ((step->step_id.step_id == SLURM_INTERACTIVE_STEP) &&
+	     !set_interactive_freq) ||
 	    (step->step_id.step_id == SLURM_EXTERN_CONT))
 		return;
 
