@@ -929,10 +929,7 @@ static void _kill_container()
 static void _run(stepd_step_task_info_t *task)
 {
 	debug3("%s: executing: %s", __func__, run_argv[2]);
-	if (oci_conf->create_env_file)
-		execve(run_argv[0], run_argv, step->env);
-	else
-		execv(run_argv[0], run_argv);
+	execv(run_argv[0], run_argv);
 	fatal("execv(%s) failed: %m", run_argv[0]);
 }
 
